@@ -1,6 +1,5 @@
-#
-#  $Id: ReadKey.pm,v 1.5 2002/03/21 07:38:21 gellyfish Exp $
-# 
+
+package Term::ReadKey;
 
 =head1 NAME
 
@@ -10,7 +9,7 @@ Term::ReadKey - A perl module for simple terminal control
 
 	use Term::ReadKey;
 	ReadMode 4; # Turn off controls keys
-	while (not defined ($key = ReadKey(-1)) {
+	while (not defined ($key = ReadKey(-1))) {
 		# No key yet
 	}
 	print "Get key $key\n";
@@ -23,7 +22,12 @@ control over terminal driver modes (cbreak, raw, cooked, etc.,) support for
 non-blocking reads, if the architecture allows, and some generalized handy
 functions for working with terminals. One of the main goals is to have the
 functions as portable as possible, so you can just plug in "use
-Term::ReadKey" on any architecture and have a good likelyhood of it working.
+Term::ReadKey" on any architecture and have a good likelihood of it working.
+
+Version 2.30.01:
+Added handling of arrows, page up/down, home/end, insert/delete keys 
+under Win32. These keys emit xterm-compatible sequences.
+Works with Term::ReadLine::Perl.
 
 =over 8
 
@@ -204,14 +208,49 @@ This call does nothing under Windows.
 
 Kenneth Albanowski <kjahds@kjahds.com>
 
-Currently maintained by Jonathan Stowe <jns@gellyfish.com>
+Currently maintained by Jonathan Stowe <jns@gellyfish.co.uk>
+
+=item SUPPORT
+
+The code is maintained at 
+
+     https://github.com/jonathanstowe/TermReadKey
+
+Please feel free to fork and suggest patches.
+
+
+=item LICENSE
+
+Prior to the 2.31 release the license statement was:
+
+ Copyright (C) 1994-1999 Kenneth Albanowski.
+                2001-2005 Jonathan Stowe and others
+
+                 Unlimited distribution and/or modification is allowed as long as this
+                  copyright notice remains intact.
+
+And was only stated in the README file.
+
+Because I believe the original author's intent was to be more open than the
+other commonly used licenses I would like to leave that in place. However if
+you or your lawyers require something with some more words you can optionally
+choose to license this under the standard Perl license:
+
+      This module is free software; you can redistribute it and/or modify it
+      under the terms of the Artistic License. For details, see the full
+      text of the license in the file "Artistic" that should have been provided
+      with the version of perl you are using.
+
+      This program is distributed in the hope that it will be useful, but
+      without any warranty; without even the implied warranty of merchantability
+      or fitness for a particular purpose.
+
 
 =cut
 
-package Term::ReadKey;
+use vars qw($VERSION);
 
-
-$VERSION = '2.19';
+$VERSION = '2.31';
 
 require Exporter;
 require AutoLoader;
@@ -223,17 +262,16 @@ use Carp;
 # Items to export into callers namespace by default
 # (move infrequently used names to @EXPORT_OK below)
 
-@EXPORT =  qw(
-	      ReadKey 
-              ReadMode 
-              ReadLine 
-              GetTerminalSize 
-              SetTerminalSize
-	      GetSpeed 
-              GetControlChars 
-              SetControlChars
-             );
-
+@EXPORT = qw(
+  ReadKey
+  ReadMode
+  ReadLine
+  GetTerminalSize
+  SetTerminalSize
+  GetSpeed
+  GetControlChars
+  SetControlChars
+);
 
 @EXPORT_OK = qw();
 
@@ -242,117 +280,162 @@ bootstrap Term::ReadKey;
 # Preloaded methods go here.  Autoload methods go after __END__, and are
 # processed by the autosplit program.
 
-
 # Should we use LINES and COLUMNS to try and get the terminal size?
 # Change this to zero if you have systems where these are commonly
-# set to erroneous values. (But if either are nero zero, they won't be
+# set to erroneous values. (But if either are near zero, they won't be
 # used anyhow.)
 
 $UseEnv = 1;
 
+$CurrentMode = 0;
 
-%modes=( original    => 0, 
-         restore     => 0, 
-         normal      => 1, 
-         noecho      => 2, 
-	 cbreak      => 3, 
-         raw         => 4, 
-         'ultra-raw' => 5);
+%modes = (
+    original    => 0,
+    restore     => 0,
+    normal      => 1,
+    noecho      => 2,
+    cbreak      => 3,
+    raw         => 4,
+    'ultra-raw' => 5
+);
 
-sub ReadMode {
-	my($mode) = $modes{$_[0]};
-	my($fh) = normalizehandle((@_>1?$_[1]:\*STDIN));
-	if(defined($mode))
-		{ SetReadMode($mode,$fh) } 
-	elsif( $_[0] =~ /^\d/)
-		{ SetReadMode($_[0],$fh) }
-	else
-		{ croak("Unknown terminal mode `$_[0]'"); }
-}
-
-sub normalizehandle {
-	my($file) = @_;
-#	print "Handle = $file\n";
-	if(ref($file)) { return $file; } # Reference is fine
-#	if($file =~ /^\*/) { return $file; } # Type glob is good
-	if (ref(\$file) eq 'GLOB') { return $file; } # Glob is good
-#	print "Caller = ",(caller(1))[0],"\n";
-	return \*{((caller(1))[0])."::$file"};
-}
-
-
-sub GetTerminalSize {
-	my($file) = normalizehandle((@_>1?$_[1]:\*STDOUT));
-	my(@results) = ();
-	my(@fail);
-	
-	if(&termsizeoptions() & 1) # VIO
-	{
-		@results = GetTermSizeVIO($file);
-		push(@fail,"VIOGetMode call");
-	} elsif(&termsizeoptions() & 2) # GWINSZ
-	{
-		@results = GetTermSizeGWINSZ($file);
-		push(@fail,"TIOCGWINSZ ioctl");
-	} elsif(&termsizeoptions() & 4) # GSIZE
-	{
-		@results = GetTermSizeGSIZE($file);
-		push(@fail,"TIOCGSIZE ioctl");
-	} elsif(&termsizeoptions() & 8) # WIN32
-	{
-		@results = GetTermSizeWin32($file);
-		push(@fail,"Win32 GetConsoleScreenBufferInfo call");
-	} else
-	{
-		@results = ();
-	}
-	
-	if(@results<4 and $UseEnv) {
-		my($C) = defined($ENV{COLUMNS}) ? $ENV{COLUMNS} : 0;
-		my($L) = defined($ENV{LINES}) ? $ENV{LINES} : 0;
-		if(($C >= 2) and ($L >=2)) {
-			@results = ($C+0,$L+0,0,0);
-		}
-		push(@fail,"COLUMNS and LINES environment variables");
-	}
-	
-	if(@results<4) {
-		my($prog) = "resize";
-		
-		# Workaround for Solaris path sillyness
-		if(-f "/usr/openwin/bin/resize") { $prog = "/usr/openwin/bin/resize"}
-		
-		my($resize) = scalar(`$prog`);
-		if($resize =~ /COLUMNS\s*=\s*(\d+)/ or 
-		   $resize =~ /setenv\s+COLUMNS\s+'?(\d+)/)  {
-			$results[0] = $1;
-			if( $resize =~ /LINES\s*=\s*(\d+)/ or
-			    $resize =~ /setenv\s+LINES\s+'?(\d+)/) {
-				$results[1] = $1;
-				@results[2,3] = (0,0);
-			} else {
-				@results = ();
-			}
-		} else {
-			@results = ();
-		}
-		push(@fail,"resize program");
-	}
-	
-	if(@results<4) {
-		die "Unable to get Terminal Size.".join("", map(" The $_ didn't work.",@fail));
-	}
-	
-	@results;
-}
-
-
-
-if(&blockoptions() & 1) # Use nodelay
+sub ReadMode
 {
-	if(&blockoptions() & 2) #poll
-	{
-		eval <<'DONE';
+    my ($mode) = $modes{ $_[0] };
+    my ($fh) = normalizehandle( ( @_ > 1 ? $_[1] : \*STDIN ) );
+    if ( defined($mode) ) { $CurrentMode = $mode }
+    elsif ( $_[0] =~ /^\d/ ) { $CurrentMode = $_[0] }
+    else { croak("Unknown terminal mode `$_[0]'"); }
+    SetReadMode($CurrentMode, $fh);
+}
+
+sub normalizehandle
+{
+    my ($file) = @_;
+
+    #	print "Handle = $file\n";
+    if ( ref($file) ) { return $file; }    # Reference is fine
+
+    #	if($file =~ /^\*/) { return $file; } # Type glob is good
+    if ( ref( \$file ) eq 'GLOB' ) { return $file; }    # Glob is good
+
+    #	print "Caller = ",(caller(1))[0],"\n";
+    return \*{ ( ( caller(1) )[0] ) . "::$file" };
+}
+
+sub GetTerminalSize
+{
+    my ($file) = normalizehandle( ( @_ > 1 ? $_[1] : \*STDOUT ) );
+    my (@results) = ();
+    my (@fail);
+
+    if ( &termsizeoptions() & 1 )                       # VIO
+    {
+        @results = GetTermSizeVIO($file);
+        push( @fail, "VIOGetMode call" );
+    }
+    elsif ( &termsizeoptions() & 2 )                    # GWINSZ
+    {
+        @results = GetTermSizeGWINSZ($file);
+        push( @fail, "TIOCGWINSZ ioctl" );
+    }
+    elsif ( &termsizeoptions() & 4 )                    # GSIZE
+    {
+        @results = GetTermSizeGSIZE($file);
+        push( @fail, "TIOCGSIZE ioctl" );
+    }
+    elsif ( &termsizeoptions() & 8 )                    # WIN32
+    {
+        @results = GetTermSizeWin32($file);
+        push( @fail, "Win32 GetConsoleScreenBufferInfo call" );
+    }
+    else
+    {
+        @results = ();
+    }
+
+    if ( @results < 4 and $UseEnv )
+    {
+        my ($C) = defined( $ENV{COLUMNS} ) ? $ENV{COLUMNS} : 0;
+        my ($L) = defined( $ENV{LINES} )   ? $ENV{LINES}   : 0;
+        if ( ( $C >= 2 ) and ( $L >= 2 ) )
+        {
+            @results = ( $C + 0, $L + 0, 0, 0 );
+        }
+        push( @fail, "COLUMNS and LINES environment variables" );
+    }
+
+    if ( @results < 4 )
+    {
+        my ($prog) = "resize";
+
+        # Workaround for Solaris path silliness
+        if ( -f "/usr/openwin/bin/resize" ) {
+            $prog = "/usr/openwin/bin/resize";
+        }
+
+        my ($resize) = scalar(`$prog 2>/dev/null`);
+        if (
+            defined $resize
+            and (  $resize =~ /COLUMNS\s*=\s*(\d+)/
+                or $resize =~ /setenv\s+COLUMNS\s+'?(\d+)/ )
+          )
+        {
+            $results[0] = $1;
+            if (   $resize =~ /LINES\s*=\s*(\d+)/
+                or $resize =~ /setenv\s+LINES\s+'?(\d+)/ )
+            {
+                $results[1] = $1;
+                @results[ 2, 3 ] = ( 0, 0 );
+            }
+            else
+            {
+                @results = ();
+            }
+        }
+        else
+        {
+            @results = ();
+        }
+        push( @fail, "resize program" );
+    }
+
+    if ( @results < 4 )
+    {
+        my ($prog) = "stty size";
+
+        my ($stty) = scalar(`$prog 2>/dev/null`);
+        if (
+            defined $stty
+            and (  $stty =~ /(\d+) (\d+)/ )
+          )
+        {
+            $results[0] = $2;
+			$results[1] = $1;
+			@results[ 2, 3 ] = ( 0, 0 );
+        }
+        else
+        {
+            @results = ();
+        }
+        push( @fail, "stty program" );
+    }
+
+    if ( @results != 4 )
+    {
+        warn "Unable to get Terminal Size."
+          . join( "", map( " The $_ didn't work.", @fail ) );
+	return undef;
+    }
+
+    @results;
+}
+
+if ( &blockoptions() & 1 )    # Use nodelay
+{
+    if ( &blockoptions() & 2 )    #poll
+    {
+        eval <<'DONE';
 		sub ReadKey {
 		  my($File) = normalizehandle((@_>1?$_[1]:\*STDIN));
                   if (defined $_[0] && $_[0] > 0) {
@@ -387,10 +470,10 @@ if(&blockoptions() & 1) # Use nodelay
                   $value;
 		}
 DONE
-	} 
-        elsif(&blockoptions() & 4) #select
-	{
-		eval <<'DONE';
+    }
+    elsif ( &blockoptions() & 4 )    #select
+    {
+        eval <<'DONE';
 		sub ReadKey {
 		  my($File) = normalizehandle((@_>1?$_[1]:\*STDIN));
                   if(defined $_[0] && $_[0]>0) {
@@ -412,8 +495,10 @@ DONE
 			$value;
 		}
 DONE
-	} else { #nothing
-		eval <<'DONE';
+    }
+    else
+    {    #nothing
+        eval <<'DONE';
 		sub ReadKey {
 		  my($File) = normalizehandle((@_>1?$_[1]:\*STDIN));
 		    if(defined $_[0] && $_[0]>0) {
@@ -457,11 +542,11 @@ DONE
 			$value;
 		}
 DONE
-	}
+    }
 }
-elsif(&blockoptions() & 2) # Use poll
+elsif ( &blockoptions() & 2 )    # Use poll
 {
-	eval <<'DONE';
+    eval <<'DONE';
 	sub ReadKey {
 	  my($File) = normalizehandle((@_>1?$_[1]:\*STDIN));
 		if(defined $_[0] && $_[0] != 0) {
@@ -478,9 +563,9 @@ elsif(&blockoptions() & 2) # Use poll
 	}
 DONE
 }
-elsif(&blockoptions() & 4) # Use select
+elsif ( &blockoptions() & 4 )    # Use select
 {
-	eval <<'DONE';
+    eval <<'DONE';
 	sub ReadKey {
 	  my($File) = normalizehandle((@_>1?$_[1]:\*STDIN));
 		if(defined $_[0] && $_[0] !=0 ) {
@@ -497,16 +582,16 @@ elsif(&blockoptions() & 4) # Use select
 	}
 DONE
 }
-elsif(&blockoptions() & 8) # Use Win32
+elsif ( &blockoptions() & 8 )    # Use Win32
 {
-	eval <<'DONE';
+    eval <<'DONE';
 	sub ReadKey {
 	  my($File) = normalizehandle((@_>1?$_[1]:\*STDIN));
-		if ($_[0]) {
+        if ($_[0] || $CurrentMode >= 3) {
 			Win32PeekChar($File, $_[0]);
-		} else {
-			getc $File;
-		}
+        } else {
+        	getc $File;
+        }
 		#if ($_[0]!=0) {return undef if !Win32PeekChar($File, $_[0])};
 		#getc $File;
 	}
@@ -522,7 +607,7 @@ DONE
 }
 else
 {
-	eval <<'DONE';
+    eval <<'DONE';
 	sub ReadKey {
 	  my($File) = normalizehandle((@_>1?$_[1]:\*STDIN));
 		if($_[0]) 
@@ -538,7 +623,7 @@ else
 DONE
 }
 
-package Term::ReadKey; # return to package ReadKey so AutoSplit is happy
+package Term::ReadKey;    # return to package ReadKey so AutoSplit is happy
 1;
 
 __END__;
